import java.util.*;

public class XMLElement { 
 private XMLElement next,prev;
 private String     name;
 private Hashtable  items,attributes;
 private Vector     itemsList,attributesList;
 private String     value;
 private int        counter;
        
 XMLElement (String name) { 
  this.name = name;
 }

 public String getName() { 
  return name;
 }
    
 final void addItem (XMLElement elem) { 
  if (items == null) { 
   items = new Hashtable();
   itemsList = new Vector();
  }
  XMLElement prev = (XMLElement)items.get(elem.name);
  if (prev != null) { 
   elem.next = null;
   elem.prev = prev.prev;
   elem.prev.next = elem;
   prev.prev = elem;
   prev.counter += 1;
  } 
  else { 
   items.put (elem.name, elem);
   elem.prev = elem;
   elem.counter = 1;
  }
  itemsList.addElement(elem);
 }
   
 final void addAttribute(String name, String value) { 
  XMLAttribute attr = new XMLAttribute(name, value);
  if (attributes == null) { 
   attributes = new Hashtable();
   attributesList = new Vector();
  }
  attributesList.addElement(attr);
  attributes.put(name, attr);
 }
   
 final Vector getItems() { 
  return itemsList;
 }

 final Vector getAttributes() { 
  return attributesList;
 }

 final XMLElement getItem(String name) { 
  if (items != null) { 
   XMLElement item = (XMLElement)items.get(name);
   if (item != null) return item;
  }
  return null;
 }
    
 final XMLElement getNextItem() { 
  return next;
 }
    
 final int getCounter() { 
  return counter;
 }
    
 final XMLAttribute getAttribute(String name) { 
  return attributes != null ? (XMLAttribute)attributes.get(name) : null;
 }
    
 final void setValue(String value) { 
  this.value = value;
 }
    
 final String getValue() { 
  return value;
 }
    
 final int getIntValue() { 
  if (value == null) { 
   throw new XMLException("Value not set");
  }
  try { 
   return Integer.parseInt(value);
  } 
  catch (NumberFormatException x) { 
   throw new XMLException("Element value '" + value + "' is not an integer constant");
  }
 }

 public static String upperStr =
  "ABWGDEVZIJKLMNOPRSTUFHCQYXŨ";
 public static String lowerStr =
  "abwgdevzijklmnoprstufhcqyx";
 public String toUpperCase(String in) {
  String out="";
  char c; 
  int n,l=in.length();
  for (int i=0; i<l; i++) {
   c=in.charAt (i);
   n=lowerStr.indexOf(c);
   if (n>-1) out+=upperStr.charAt(n);
   else out+=c;
  }
  return out;
 }

 public Vector search(String pattern) {
  Vector result = new Vector();
  pattern = toUpperCase(pattern.trim());
  if (itemsList != null) { 
   int n=itemsList.size();
   //for (int i = n; --i >= 0;) { 
   for (int i = 0; i<n; i++) { 
    XMLElement elem = (XMLElement)itemsList.elementAt(i);
    if (elem.searchItems(pattern)) { 
     result.addElement(elem);
    }
    if (i%10==0) SearchBox.box.setTitle (SearchBox.boxHdr + ": " + ((i*100)/n)+"%");
   }
   SearchBox.box.setTitle (SearchBox.boxHdr);
  }
  return result;
 }

 private boolean searchItems (String pattern) {
  if (toUpperCase(name).equals(pattern)) { 
   return true;
  }
  if (value != null) { 
   if (toUpperCase(value).indexOf(pattern) >= 0) {
    return true;
   }
  }
  if (attributesList != null) { 
   for (int i = attributesList.size(); --i >= 0;) { 
    XMLAttribute attr = (XMLAttribute)attributesList.elementAt(i);
    if ((attr.getName() != null && toUpperCase(attr.getName()).equals(pattern)) 
    || (attr.getValue() != null && toUpperCase(attr.getValue()).indexOf(pattern) >= 0)) {
     return true;
    }
   }
  }
  if (itemsList != null) { 
   for (int i = itemsList.size(); --i >= 0;) { 
    XMLElement elem = (XMLElement)itemsList.elementAt(i);
    if (elem.searchItems(pattern)) { 
     return true;
    }
   }
  }
  return false;
 }

}        

